;Beispiel3 zur Anwendung von AssembleIt2
;es werden AutoIt-Variablen im Assemblercode verwendet
;einfachste Variante, auch keine Parameterbergabe per Stack bentigt!

;
;Fr 32Bit-Modus mus gesetzt werden #AutoIt3Wrapper_UseX64=n
;Fr 64Bit-Modus mus gesetzt werden #AutoIt3Wrapper_UseX64=y


#AutoIt3Wrapper_UseX64=n                                 ;32Bit-Modus

#include-once
#include <assembleit2_64.au3>
#include <SliderConstants.au3>


#cs
    Beispielscript um eine mit ASM-Code erstellte Farbpalette in einer Bitmap darzustellen
    Innerhalb von 2 Schleifen werden die Grn- und Blauwerte von Pixeln gesetzt,
    der Rot-Anteil des Pixels wird durch einen Slider in der GUI festgelegt und an den Code bergeben.
    Weiterhin wird die Startadresse der Bitmap an den Code bergeben.

    Vorgestellt sind mehrere Varianten, um ASM-Code mittels Assembleit2 ablaufen zu lassen.

    1. Beispiel zur Verwendung von Macros und in ASM definierten Variablen (kein DllCallAddress!)
    2. Beispiel als auch per DllCallAddress ausfhrbaren Code (sehr schnell!)
    3. Beispiel mit Verwendung von AutoIt-Variablen im ASM-Code (kein DllCallAddress!)

    Der Overhead von mit AssembleIt2 laufenden Scripten betrgt nur eine Handvoll Millisekunden (2-3ms)!
    In dieser Zeit wird der ASM-Code zur Laufzeit assembliert, in den Speicher geschrieben und dort per DllCallAddress ausgefhrt.

    AssembleIt2 ist auch in der Lage, den Code zu assemblieren und in einer Variable oder Datei abzuspeichern.
    Diesen Code kann man in einer Struct speichern um ihn per DllCallAddress() direkt im Speicher auszufhren.
    Dies ist die zzt. schnellste Variante, um Assemblerprogramme auszufhren. AssembleIt2 und smtliche dafr
    notwendigen Includes sind nicht mehr ntig!

    Bei extrem aufwendigen Berechnungen bleibt daher abzuwgen, ob man die Vorteile der Verwendung von
    Macros und (AutoIt-)Variablen im Code der Erstellung von mit DllCallAddress()lauffhigen Code vorzieht!


    Vom assemblierten Code wird die Anzahl der Prozessortakte zurckgegeben. Bitte beachten, dass die Laufzeit des Codes
    so gering ist, dass moderne Prozessoren im Sleep-Modus bleiben und daher nur mit minimalem Takt laufen!

#ce


;###################################################################################################
;Beispiel3 mit im ASM-Code enthaltenen AutoIt-Variablen $ptr und $sliderpos
;"ausfhrlicher" ASM-Code
;###################################################################################################

#cs _farbverlauf
    use32

    mov edi,$ptr                                         ;Pointer auf Startadresse Bitmap  AUTOITVARIABLE

    rdtsc                                                ;Ticks1 = edx:eax um Takte zu zhlen ^^
    push eax                                             ;Ticks1 auf Stack speichern

    mov ecx,0                                            ;x=0
    _xschleife:                                          ;for x=0 to 255  grn

    mov ebx,0                                            ;y=0
    _yschleife:                                          ;for y=0 to 255   ;blau

    mov eax,0xFF                                         ;000000AA  alpha
    shl eax,8                                            ;0000AA00
    or eax,    $sliderpos                                ;0000AARR  rot  AUTOITVARIABLE
    shl eax,8                                            ;00AARR00
    or eax,ecx                                           ;00AARRGG  grn
    shl eax,8                                            ;AARRGG00
    or eax,ebx                                           ;AARRGGBB  blau
    mov [edi],eax                                        ;pixel an adresse schreiben
    add edi,4                                            ;nchstes pixel

    inc ebx                                              ;blau=blau+1
    cmp ebx,256                                          ;ist blau=256?
    jne _yschleife                                       ;nein, dann nchstes blau

    inc ecx                                              ;grn=grn+1
    cmp ecx,256                                          ;ist grn=256
    jne _xschleife                                       ;nein, dann nchstes grn

    rdtsc                                                ;Ticks= edx:eax
    pop ebx                                              ;Ticks1
    sub eax,ebx                                          ;Ticks=Ticks-Ticks1 Anzahl bentigter Takte seit Start zurckgeben

    ret                                                  ;Ende
#ce



$gui = GUICreate("Beispiel", 296, 256)

$slider = GUICtrlCreateSlider(261, 5, 30, 251, BitOR($TBS_VERT, $TBS_TOP, $TBS_LEFT, $TBS_BOTH, $TBS_NOTICKS, $TBS_ENABLESELRANGE))
GUICtrlSetLimit(-1, 255, 0)

GUISetState()

$DC_gui = _WinAPI_GetDC($gui)                            ;DeviceContext der GUI holen
Global $ptr, $hbmp                                       ;pointer auf die pixel, handle bitmap
$DC_bitmap = _CreateNewBmp32(256, 256, $ptr, $hbmp)      ;bitmap erstellen


While GUIGetMsg() <> -3

    $sliderpos = GUICtrlRead($slider)                    ;sliderposition

    ;Assembleit benutzen, assembliert code whrend der Laufzeit JIT
    ;wenn die Parameter als Autoit-Variablen verwendet werden, sind keine Parameter ntig!
    $t = TimerInit()
    Local $ret = _AssembleIt2("uint_ptr", "_farbverlauf") ;32Bit-Modus
    ConsoleWrite("_AssembleIt2()       Anzahl Takte = " & ($ret) & "     Zeit [ms] = " & Int(1000 * TimerDiff($t)) / 1000 & @CRLF) ;### Debug Console

    _WinAPI_BitBlt($DC_gui, 0, 0, 256, 256, $DC_bitmap, 0, 0, 0xCC0020);$srccopy

WEnd


_DeleteBitmap32($DC_bitmap, $ptr, $hbmp)



Func _CreateNewBmp32($iwidth, $iheight, ByRef $ptr, ByRef $hbmp) ;erstellt leere 32-bit-Bitmap; Rckgabe $HDC und $ptr und handle auf die Bitmapdaten
    $hcdc = _WinAPI_CreateCompatibleDC(0)                ;Desktop-Kompatiblen DeviceContext erstellen lassen
    $tBMI = DllStructCreate($tagBITMAPINFO)              ;Struktur der Bitmapinfo erstellen und Daten eintragen
    DllStructSetData($tBMI, 1, DllStructGetSize($tBMI) - 4);Structgre abzglich der Daten fr die Palette
    DllStructSetData($tBMI, 2, $iwidth)
    DllStructSetData($tBMI, 3, -$iheight)                ;minus =standard = bottomup
    DllStructSetData($tBMI, 4, 1)
    DllStructSetData($tBMI, 5, 32)                       ;32 Bit = 4 Bytes => AABBGGRR
    $adib = DllCall('gdi32.dll', 'ptr', 'CreateDIBSection', 'hwnd', 0, 'ptr', DllStructGetPtr($tBMI), 'uint', 0, 'ptr*', 0, 'ptr', 0, 'uint', 0)
    $hbmp = $adib[0]                                     ;hbitmap handle auf die Bitmap, auch per GDI+ zu verwenden
    $ptr = $adib[4]                                      ;pointer auf den Anfang der Bitmapdaten, vom Assembler verwendet
    _WinAPI_SelectObject($hcdc, $hbmp)                   ;objekt hbitmap in DC
    Return $hcdc                                         ;DC der Bitmap zurckgeben
EndFunc                                                  ;==>_CreateNewBmp32

Func _DeleteBitmap32($DC, $ptr, $hbmp)
    _WinAPI_DeleteDC($DC)
    _WinAPI_DeleteObject($hbmp)
    $ptr = 0
EndFunc                                                  ;==>_DeleteBitmap32





























